package cami;

import java.util.StringTokenizer;
import java.util.Vector;

/**
 * Ensemble d'outils pour la manipulation de commandes CAMI.<br>
 * On trouvera entre autre :
 * <ul>
 * <li> Un parseur d'entier</li>
 * <li> Un parseur de chaine de caracteres</li>
 * <li> Un producteur de chaines conformes CAMI</li>
 * </ul>
 */

public class CamiParser {

	private String message;


	public CamiParser(String commandeCAMI) {
		this.message = commandeCAMI.substring(3);
	}

	/**
	 * Retourne l'entier CAMI du message CAMI
	 *
	 * @param token Le separateur
	 * @return l'entier CAMI decouvert (sous forme de chaine de caracteres)
	 * 	Si aucun entier n'est trouve... La methode renvoie la chaine vide
	 */
	public final String parseInt(String token) {

		String res = "";
		int cpt = 0;

		if (!(this.message.subSequence(cpt, cpt + 1)).equals(token)) {
			while (!(this.message.subSequence(cpt, cpt + 1)).equals(token)) {
				res += this.message.subSequence(cpt, cpt + 1);
				cpt++;
			}
			cpt++;
			this.message = this.message.substring(cpt);

			return res;
		} else {
			this.message = this.message.substring(1);

			return "";
		}
	}

	public final Vector<String> parse () {
		Vector<String> v = new Vector<String>();

		int n = 0;
		boolean inInt = false;
		boolean afterComma = false;
		for (int i = 0;  i<message.length() ; i++) {
			char c = message.charAt(i);
			if (c >= '0' && c <= '9') {
				if (! inInt) {
					inInt = true;
					n=0;
				}
				n *= 10;
				n += c - '0';
			} else if (c == ':') {
				assert(inInt);
				v.add( message.substring(i+1, i+n+1));
				i += n+1;
				inInt=false;
			} else if (c == ')')  {
				if (inInt) {
					v.add(Integer.toString(n));
					inInt = false;
				} else if (afterComma) {
					v.add(null);
				}
			} else if (c == ',') {
				if (inInt) {
					v.add(Integer.toString(n));
					inInt = false;
				} else {
					v.add(null);
					afterComma = true;
				}
			}
		}


		return v;
	}

	/**
	 * Retourne la chaine CAMI du message CAMI
	 *
	 * @param token Le separateur
	 * @return la chaine CAMI
	 */
	public final String parseString(String token) {

		StringTokenizer st = new StringTokenizer(this.message);
		String tailleChaine;
		int taille;
		String res = "";

		if (!(this.message.subSequence(0, 1)).equals(token)) {

			// Recupere la taille indiquee par la commande CAMI
			tailleChaine = st.nextToken(":");
			taille = Integer.parseInt(tailleChaine);

			// Recupere le texte ecrit apres la taille (et les :) jusqu'au bout
			res = this.message.substring(tailleChaine.length() + 1, tailleChaine.length() + 1 + taille);
			this.message = this.message.substring(tailleChaine.length() + 1 + taille + 1); // ??

			return res;
		} else {
			this.message = this.message.substring(1);

			return "";
		}
	}

	/**
	 * Conversion d'une chaine de carateres en message CAMI.<br>
	 * En CAMI chaque chaine de carateres est precede de sa taille et de ':'
	 *
	 * @param s la chaine a convertir
	 * @return la chaine convertie
	 */
	public static String stringToCAMI(String s) {

		if (s!= null &&  !s.equals("") ) {
			String cami = "" + s.length() + ":" + s;

			return cami;
		} else {

			return "";
		}
	}
}
