package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StreamCorruptedException;
import java.net.InetAddress;
import java.net.UnknownHostException;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;

public class ConfigurationGUI extends JFrame  {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 317831722756700749L;
	private JMenuBar mb;
	private JMenuItem mi_open;
	private JMenuItem mi_save;
	private JMenuItem mi_save_as;
	private JTextArea error_text;

	private NodeConfigPanel config_panel;
	private HostTablePanel host_panel;
	private JMenuItem mi_new;
	
	class MenuAL implements ActionListener {

		private String currentSavePath= "default.dmc";

		public void actionPerformed(ActionEvent e) {
			Object source = e.getSource();
			if (source == mi_open) {
				JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
				int returnVal = chooser.showOpenDialog(mb);
				if(returnVal == JFileChooser.APPROVE_OPTION) {
					String path = null ;
					try {
						path = chooser.getSelectedFile().getCanonicalPath();
						System.out.println("You chose to open this file: " + path);
						host_panel.getConf().loadConfiguration(path);
						System.out.println("new Configuration loaded : "+Configuration.getConf().asString());
						updateConfiguration();
						currentSavePath = path ;
					} catch (FileNotFoundException e1) {
						error_text.setText( "Configuration load failed due bad file name error :"+path+"\n"+e1);
					} catch (StreamCorruptedException e2) {
						error_text.setText( "Configuration load failed due to bad file format. \n Are you sure this is a save file ? ");
					} catch (IOException e3) {						
						error_text.setText( "Configuration load failed due to exception : " +e3);
					}
				}				
			} else if (source == mi_save || source == mi_save_as) {
				String path = currentSavePath;
				try {
					if (source == mi_save_as || path.equals("")) {					
						JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
						int b = chooser.showOpenDialog(mb);
						if (b == JOptionPane.OK_OPTION) 
							path = chooser.getSelectedFile().getCanonicalPath();
						else
							return ;
					}
					System.out.println("Saving to file: " + path);
					host_panel.getConf().saveConfiguration(path);
					System.out.println("Configuration saved :"+host_panel.getConf().asString());
					currentSavePath = path;
				} catch (FileNotFoundException e1) {
					error_text.setText( "Configuration save failed due bad file name error :"+path+"\n"+e1);
				} catch (IOException e3) {
					error_text.setText( "Configuration save failed due to exception : " +e3);
				}
			} else if (source == mi_new) {
				currentSavePath = "";
				host_panel.setConf(new Configuration());
			}
		}

	}	
	
	void updateConfiguration() {
		host_panel.updateConfiguration();
		config_panel.updateConfiguration();
	}
	
	


	
	public ConfigurationGUI () {
		super("libdmc Configuration Tool");
		setSize(800,600);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		
		System.err.println(Configuration.getConf().asString());
		JPanel main_panel = new JPanel(new BorderLayout());
		
		MenuAL menu_al = new MenuAL();
		JMenu menu_File = new JMenu("File");
		{
			mi_new = new JMenuItem("New");
			mi_new.addActionListener(menu_al);
			menu_File.add(mi_new);
			menu_File.addSeparator();
			
			mi_open = new JMenuItem("Open...");
			mi_open.addActionListener(menu_al);
			menu_File.add(mi_open);
			menu_File.addSeparator();
		
			mi_save = new JMenuItem("Save");
			mi_save.addActionListener(menu_al);
			menu_File.add(mi_save);
			mi_save_as = new JMenuItem("Save as...");
			menu_File.add(mi_save_as);
			mi_save_as.addActionListener(menu_al);
			
		}
		mb = new JMenuBar();
		mb.add(menu_File);
		main_panel.add("North",mb);		
		
		error_text = new JTextArea(5,80);
		error_text.setLineWrap(true);
		error_text.setBackground(Color.LIGHT_GRAY);
		
		main_panel.add("South",error_text);
		
		host_panel = new HostTablePanel();

		

		
		//add(host_panel);
		config_panel = new NodeConfigPanel() ;
		
		
		//add(config_panel);
		//JSplitPane panels = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		//panels.add(host_panel);
		//panels.add(config_panel);
		//main_panel.add("Center",panels);
		main_panel.add("Center",host_panel);
		//main_panel.add("East",config_panel);
//		main_panel.add("East",button_panel);
		
		updateConfiguration();
		add(main_panel);
		
	}
	

	
	public static void main(String[] args) {
		try {
			Configuration c = Configuration.getConf();
			c.addHostIP(InetAddress.getLocalHost().getHostAddress());
			c.addHostIP("google.fr");
			c.addHostIP("phoenix.lip6.fr");
			System.out.println(c.getHostIP(2));
		} catch (UnknownHostException e) {
			System.err.println( "Unknown host :"+e);
			e.printStackTrace();
		}
		new ConfigurationGUI ().setVisible(true);		
	}



}
