package gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.net.UnknownHostException;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;

public class HostTablePanel extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1895670316658099046L;
	private boolean DEBUG = false;
	private gui.HostTablePanel.HostListModel hlm;
	private Configuration conf;
	private JButton addHostButton;
	
	
	protected static String[] columnToolTips = {
		"The network name of the node",
		"The network address of the node",
		"The port of the state manager service this node offers",
		"The port of the firing manager service this node offers",
		"The port of the activity monitor service this node offers",
		"The port of the new state stack service this node offers",
		"The local deployment path"
		   };
    
	
	@SuppressWarnings("serial")
	public HostTablePanel () {
		super(new BorderLayout());
	
		conf = Configuration.getConf();
		hlm = new HostListModel();	

		// for sorting by columns
		TableSorter sorter = new TableSorter(hlm);
		JTable configTable = new JTable(sorter) {
			//			Implement table header = column tool tips.
	           protected JTableHeader createDefaultTableHeader() {
	                return new JTableHeader(columnModel) {
	                    public String getToolTipText(MouseEvent e) {
	                        java.awt.Point p = e.getPoint();
	                        int index = columnModel.getColumnIndexAtX(p.x);
	                        int realIndex = columnModel.getColumn(index).getModelIndex();
	                        return columnToolTips[realIndex];
	                    }
	                };
	            }
		};             
		sorter.setTableHeader(configTable.getTableHeader());
		
		configTable.setPreferredScrollableViewportSize(new Dimension(500, 70));	      			
		
		JScrollPane scrollPane = new JScrollPane(configTable);
		add("Center",scrollPane);
		
		
		JPanel button_panel = new JPanel();
		addHostButton = new JButton("Add host");
		addHostButton.addActionListener(this);
		button_panel.add(addHostButton);
		add("South",button_panel);
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == addHostButton) {
			int b = JOptionPane.showConfirmDialog(null, "Are you sure you want add host ?");
			if (b == JOptionPane.OK_OPTION) {
				try {
					conf.addHostIP("localhost");
				} catch (UnknownHostException e1) {
					JOptionPane.showMessageDialog(null, "Unknown host :"+e);
				}
				updateConfiguration();
			}
		}
	}
	
	public void updateConfiguration() {
		hlm.fireTableDataChanged();
	}
	
	class HostListModel extends AbstractTableModel {
	
		public static final int HOST_NAME = 0;
		public static final int HOST_IP = 1;
		public static final int HOST_SM = 2;
		public static final int HOST_FM = 3;
		public static final int HOST_AM = 4;
		public static final int HOST_NSS = 5;
		public static final int HOST_PATH = 6;

		
		public Object getValueAt(int row, int col) {
			switch (col) {
			case HOST_NAME :
				return conf.getNodeConf(row).getIP().getHostName();
			case HOST_IP :
				return conf.getNodeConf(row).getIP().getHostAddress();
			case HOST_SM :
				return Integer.toString(conf.getNodeConf(row).getSmPort());
			case HOST_FM :
				return Integer.toString(conf.getNodeConf(row).getFmPort());
			case HOST_AM :
				return Integer.toString(conf.getNodeConf(row).getAmPort());
			case HOST_NSS :
				return Integer.toString(conf.getNodeConf(row).getNssPort());
			case HOST_PATH :
				return conf.getNodeConf(row).getLocalPath();				
			}
			return "null";
		}

		public int getColumnCount() {
			return 7;
		}

		public int getRowCount() {
			return conf.getNbNodes();
		}

		public String getColumnName(int c) {
			switch (c) {
			case HOST_NAME :
				return "name";
			case HOST_IP :
				return "ip";
			case HOST_SM :
				return "SM port";
			case HOST_FM :
				return "FM port";
			case HOST_NSS :
				return "NSS port";
			case HOST_AM :
				return "AM port";			
			case HOST_PATH :
				return "Local Path";			
			}
			return ("");
		}
		
		
        public boolean isCellEditable(int row, int col) {
                return true;
        }
        
        public void setValueAt(Object value, int row, int col) {
            if (DEBUG) {
                System.out.println("Setting value at " + row + "," + col
                                   + " to " + value
                                   + " (an instance of "
                                   + value.getClass() + ")");
            }
            if (value instanceof String) {
            	String val = (String) value;
           	
            	NodeConfiguration nc = conf.getNodeConf(row);
            	switch (col) {
            	case HOST_NAME :
            	case HOST_IP :
            		try {
							nc.setHostName(val);
						} catch (UnknownHostException e) {
							JOptionPane.showMessageDialog(null, "Unknown host :"+e);
						} 
						break;
            	case HOST_SM :
            		nc.setSmPort(Integer.parseInt(val));
            		break;
            	case HOST_FM :
            		nc.setFmPort(Integer.parseInt(val));
            		break;
            	case HOST_AM :
            		nc.setAmPort(Integer.parseInt(val));
            		break;
            	case HOST_NSS :
            		nc.setNssPort(Integer.parseInt(val)); 
            		break;
            	case HOST_PATH :
            		nc.setLocalPath(val); 
            		break;
            	}            
            }
			
			fireTableCellUpdated(row, col);

            if (DEBUG) {
                System.out.println("New value of data:");
                //printDebugData();
            }
			fireTableDataChanged();
		}

	}

	public Configuration getConf() {
		return conf;
	}

	public void setConf(Configuration configuration) {
		conf = configuration;
		hlm.fireTableDataChanged();
	}
}
