package interfaces;


abstract public class AbstractController implements IController {
	abstract protected IStateManager createStateManager();
	abstract protected IStateSet createStateSet() ;
	abstract protected IFiringManager createFiringManager() ;

	protected IModelFactory mf_;
	protected IFiringManager fm_;
	protected IStateManager sm_;
	protected IActivityMonitor am_;
	protected IStateSet nss_;
	protected StatAggregator stats_;

	protected AbstractController (IModelFactory mf) {
		mf_ = mf ;
		stats_ = new StatAggregator ();
	}

	protected void initialize () {
		nss_ = createStateSet ();
		sm_ = createStateManager();
		fm_ = createFiringManager();
		am_ = createActivityMonitor ();
	}

	protected IActivityMonitor createActivityMonitor() {
		return nss_;
	}
	public void generateStateSet () {		
		fm_.processInitialState();		
		fm_.startGeneration();
		try {
			am_.waitUntilInactive();
		} catch (InterruptedException e) {
			System.err.println("State generation interrupted. "+ e + "\n");
			e.printStackTrace();
		}
	}

	public String getStatistics () {
		stats_.clear();
		mf_.getStats(stats_);
		fm_.getStats(stats_);
		sm_.getStats(stats_);
		nss_.getStats(stats_);
		return stats_.toString();		
	}
	
	public void close() {
		fm_.close();
		sm_.close();
		nss_.close();
		am_.close();
	}
}
