package libpetri.ami;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Collection;
import java.util.Vector;

import libpetri.graphics.AnnotationGraphics;
import libpetri.graphics.ArcGraphics;
import libpetri.graphics.NodeGraphics;
import libpetri.nets.ISymmetricNet;
import libpetri.structure.ITransition;
import libpetri.structure.Visibility;
import libpetri.structure.arcs.ArcType;
import transform.Trace;
import cami.CAMI;
import cami.FKCommand;

public class AMInet implements ISymmetricNet {

	// to store the named element to node number for cross references (arcs)
	private Trace<String,Integer> trace;
	// macao nodes index start from 2. Node 1 is declaration node.
	int lastIndex = 1;
	
	// storage representation as a set of Cami commands
	Vector<FKCommand> net;
	
	public AMInet() {
		trace = new Trace<String, Integer>();
		net = new Vector<FKCommand>();
	}
	
	public void writeNet (String path) throws IOException {
		PrintStream out = new PrintStream (new FileOutputStream(path));

		// NO DB/FB with FK_microPilot 
		for (FKCommand c : net) {
			out.println(c);
		}
		out.flush();
		out.close();
	}
	
	public boolean createPlace(String p) {
		trace.addTrace(p,++lastIndex);
		net.add(new FKCommand(CAMI.CN,"place",lastIndex));
		setNodeName (p,lastIndex);
		return true;
	}

	public boolean createPlace(String p,String dom) {
		createPlace(p);
		net.add(new FKCommand(CAMI.CT,"domain",lastIndex,dom));
		return true;
	}

	public boolean createTransition(String t, Visibility vis) {
		if (vis == Visibility.PUBLIC) {
//			System.err.println("WARNING : AMI net don't support public transitions.");
		}
		trace.addTrace(t,++lastIndex);
		net.add(new FKCommand(CAMI.CN,"transition",lastIndex));
		setNodeName (t,lastIndex);	
		return true;
	}
	
	public boolean createArc(String pname, String tname, ArcType at, int value) {
		return createArc(pname,tname,at, Integer.toString(value));
	}
	
	public boolean setPlaceState(String name, int mark) {
		return setPlaceState(name,Integer.toString(mark));
	}
	
	public boolean createDomain(String domName, String domElts) {
		throw new NullPointerException ("UNIMPLEMENTED !!");
	}
	
	public boolean createArc (String pname, String tname, ArcType at) {
		return createArc(pname,tname,at,1);
	}
	
	public boolean createArc(String pname, String tname, ArcType at,String func) {
		++lastIndex;
		 Integer pnum = trace.getTarget(pname);
		 Integer tnum = trace.getTarget(tname);
		 if (pnum == null) {
			 System.err.println("Error : could not find place "+pname 
					 +" while processing createArc");
			 return false;
		 }
		 if (tnum == null) {
			 System.err.println("Error : could not find transition "+tname 
					 +" while processing createArc");
			 return false;
		 }
		 FKCommand ca ;
		switch (at) {
		case INPUT :
			ca = new FKCommand(CAMI.CA,"arc",lastIndex,pnum,tnum);
			break;
		case OUTPUT :
			ca = new FKCommand(CAMI.CA,"arc",lastIndex,tnum,pnum);
			break;
		default :
			System.err.println("unknown arctype "+at+ " encountered during export to AMI net. Skipping.");				
			return false;
		}		
		net.add(ca);
		net.add(new FKCommand(CAMI.CT,"valuation",lastIndex,func));
		return true;
	}

	void setNodeName(String p, int macao) {
		net.add(new FKCommand( CAMI.CT, "name",macao,cleanup(p)));
	}



	private static String cleanup(String p) {
		// create a C identifier against sandrine/matrice warnings
		p =  p.replaceAll("\\s", "");
		p =  p.replaceAll("\\W", "_");
		p = "_" + p;
		return p;
	}

	static String macaoString(String string) {
		return string.length() + ":"+string;
	}

	public boolean setPlaceState(String pname, String s) {
		if (s != null && ! s.matches("^\\s*$"))
			net.add(new FKCommand(CAMI.CT,"marking",trace.getTarget(pname),s));
		return true;
	}

	public Collection<ITransition> getPublicTransitions() {
		// no public in ami nets !
		return new Vector<ITransition> ();
	}

	public boolean createTypeDef(String typename, String clinames) {
		throw new NullPointerException ("UNIMPLEMENTED !!");
	}

	public boolean createArc(String pname, String tname, ArcType at, String func, ArcGraphics g) {
		return createArc(pname, tname, at, func);
	}

	public boolean createArc(String pname, String tname, ArcType at, int value, ArcGraphics g) {
		return createArc(pname, tname, at, value);
	}

	public AnnotationGraphics getNodeAnnotationGraphics(String nodeName, String attribName, AnnotationGraphics g) {
		// TODO Auto-generated method stub
		return null;
	}

	public NodeGraphics getNodeGraphics(String nodeName) {
		// TODO Auto-generated method stub
		return null;
	}

	public void setNodeAnnotationGraphics(String nodeName, String attribName, AnnotationGraphics g) {
		// TODO Auto-generated method stub
		
	}

	public void setNodeGraphics(String nodeName, NodeGraphics g) {
		// TODO Auto-generated method stub
		
	}

	
	public Trace<String, Integer> getTrace() {
		return trace;
	}


}
