package libpetri.structure;

import java.util.Collection;
import java.util.Vector;

import libpetri.tensor.ITensorIterator;
import libpetri.tensor.Tensor;


public class BindingSet {

	Tensor t;
	int [] adapter;
	
	public BindingSet(Collection<Variable> vars) {
		int []dims  = new int[vars.size()];
		int i=0;
		for (Variable v : vars) {
			dims[i++]=v.getType().size();
		}
		t = new Tensor(dims);
		t.setAll();
		adapter = null;
	}
	
	
	public String toString(Vector<Variable> vars) {
		StringBuffer sb = new StringBuffer(256);
		ITensorIterator it = t.getIterator();
		while (it.hasNext()) {
			sb.append("< ");
			int []bind = it.next();
			for (int i=0;i<bind.length;i++) {
				sb.append(vars.get(i).getName()+"="+vars.get(i).getType().getByIndex(bind[i])+", ");
			}
			sb.deleteCharAt(sb.length()-2);
			sb.append(">  ");
		}
		return sb.toString();
	}

	public void setAdapter(int[] adapter) {
		this.adapter = adapter;
	}

	public ITensorIterator getIterator() {
		if (adapter == null)
			return t.getIterator();
		else 
			return new AdaptedBindingIterator (adapter,t.getIterator());
	}


	public boolean isEmpty() {
		return t.isEmpty();
	}
}
