package libpetri.structure;

import java.util.Enumeration;
import java.util.Vector;

import libpetri.nets.ISymmetricNet;
import libpetri.structure.arcs.Arc;
import libpetri.structure.state.State;
import libpetri.tensor.ITensorIterator;


public class Transition extends Node implements ITransition {

	private final Visibility vis;
	private Vector<Variable> param;
	
	public Transition(String name, Visibility vis) {
		super(name);
		this.vis = vis;
		this.param = new Vector<Variable> ();
	}

	public boolean isEnabled (State s) {
		//TODO : create bindings
		int [] bindings = new int[0];
		return arcs.isEnabled(s,bindings);
	}
	
	// TODO : reduce potential using the guard
	public BindingSet getPotentialBindings() {
		return new BindingSet(param);
	}

	public BindingSet isEnabled(State s, BindingSet bs) {
		int [] bind;
		ITensorIterator it = bs.getIterator();
		while (it.hasNext()) {
			bind=it.next();
			if (! arcs.isEnabled(s,bind)) {
				it.clear();
			}
		}
		return bs;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("\n\t");
		switch (vis) {
		case PRIVATE : sb.append("private "); break;
		case PUBLIC : sb.append("public ");
		}
		sb.append("transition "+name);
		if (! param.isEmpty()) {
			sb.append("(");
			for (Variable v : param) {
				sb.append(v+",");
			}
			sb.deleteCharAt(sb.length()-1);
			sb.append(")");
		}
		sb.append(" {");
		for (Arc a: arcs) {
			sb.append(a);
		}
		sb.append("\n\t}");
		return sb.toString();
	}

	public Enumeration<State> getSuccessors(State s, BindingSet bs) {
		return new SuccIterator(this,bs,s);
	}
	
	public void fire(State s2,int [] bindings) {
		arcs.fire(s2,bindings);
	}

	public Visibility getVis() {
		return vis;
	}

	public void flatten(ISymmetricNet receiver,String alias) {
			if (alias == null) {
				alias = getFullyQualified_Name();
				receiver.createTransition(alias, Visibility.PRIVATE);
			}
			arcs.flatten(receiver,alias);
	}

	public void convert(ISymmetricNet a) {
		a.createTransition(name, vis);
		for (Arc arc: arcs) {
			arc.convert(a);
		}

	}

	public void addParameter(Variable variable) {
		param.add(variable);
		variable.setIndex(-param.size());
	}
	
	public int getVarIndex (String name) {
		for (Variable v : param) {
			if (v.getName().equals(name))
				return v.getIndex();
		}
		return 0;
	}

	public Variable getVarByIndex(int id) {
		for (Variable v : param) {
			if (v.getIndex() == id )
				return v;
		}
		return null;
	}

}
