package libpetri.structure;

import java.util.Vector;

import libpetri.graphics.ArcGraphics;
import libpetri.namespaces.Namespace;
import libpetri.namespaces.NamingException;
import libpetri.nets.INetType;
import libpetri.structure.arcs.Arc;
import libpetri.structure.arcs.ArcType;
import libpetri.structure.arcs.Function;
import libpetri.structure.arcs.ParseFunction;
import libpetri.structure.state.ComposedState;
import libpetri.structure.state.StateFactory;

public class UnitaryPTNetType extends Namespace implements INetType {

	private Vector<Place> places = new Vector<Place>();
	private Vector<Transition> transitions = new Vector<Transition>();
	private ComposedState<Integer> initialState = StateFactory.createPTState(places.size());

	public UnitaryPTNetType(String name) {
		super(name);
	}

	public boolean createArc(String pname, String tname, ArcType at, String fun, ArcGraphics g) {
		Place p = null;
		Transition t= null;
		try {
			p = (Place) find(pname);
			if (p == null) {
				System.err.println("Unknown place "+pname+ " when treating createArc("+pname+","
						+tname+","+at+","+fun+")");
				return false;
			}
			t = (Transition) find(tname);
			if (t == null) {
				System.err.println("Unknown transition "+tname+ " when treating createArc("+pname+","
						+tname+","+at+","+fun+")");
				return false;
			}
			Function f = ParseFunction.parse(p,t,fun);
			Arc a = Arc.createArc(p, t, at,f);
			a.setGraphics(g);
			t.addArc(a);
			p.addArc(a);
			return true;
		} catch (ClassCastException e) {
			System.err.println ("Failed to create arc linking place : "+pname 
					+ " to trans :"+tname + "\n in fact p="+p + " t="+ t);
		}
		return false;	
	}

	public boolean createArc(String pname, String tname, ArcType at, String fun) {
		return createArc(pname, tname, at, fun, null);
	}

	public boolean createPlace(String name) {				
		Place p= new Place(name);			
		return addPlace(p);
	}

	private boolean addPlace (Place p) {
		try {				
			super.add(p);
			places.add(p);
			p.setIndex(places.size()-1);
			//ensure capacity
			setPlaceState(p.getName(), "0");
			return true;
		} catch (NamingException e) {
			System.err.println("Naming error !"+e);
			e.printStackTrace();
			return false;
		}
	}

	/* (non-Javadoc)
	 * @see structure.IPTNet#createTransition(java.lang.String, structure.Visibility)
	 */
	public boolean createTransition(String name, Visibility vis) {
		try {
			Transition t= new Transition(name,vis);			
			super.add(t);
			transitions.add(t);
			t.setIndex(transitions.size()-1);
			return true;
		} catch (NamingException e) {
			System.err.println("Naming error !"+e);
			e.printStackTrace();
			return false;
		}
	}

	public final ComposedState<Integer> getInitialState() {
		initialState = StateFactory.unique(initialState,this);
		return initialState;
	}

	public boolean setPlaceState(String name, String mark) {
		Place p = (Place) find(name);		
		if (p==null) {
			System.err.println("Error while treating setPlaceState("+name+","+mark+") : no place named "+name+" found");
			return false;
		}
		int i = Integer.parseInt(mark);
		// PlaceState f = ParseMarking.parse(p,mark);
		//	System.out.println(f.toString(p.getDom()));
		initialState.addToComponentState(p, i);

		return true;
	}

}
