package libpetri.structure.state;


import libpetri.nets.Indexable;


public class PTPlaceStateVector implements ComposedState<Integer> {
	private int[] marks;
	
	public PTPlaceStateVector() {
		this(0);
	}
	
	public PTPlaceStateVector(int size) {
		marks = new int[size];
	}
	
	private PTPlaceStateVector(int [] marks) {
		this.marks = marks;
	}

	public void addToComponentState(Indexable part, Integer s) {
		addToComponentState(part.getIndex(), s);
	}

	public void addToComponentState(int partIndex, Integer s) {
		ensureCapacity(partIndex);
		marks[partIndex] +=  s ;
	}

	public ComposedState<Integer> copy() {
		return new PTPlaceStateVector (marks.clone());
	}

	public Integer getComponentState(Indexable part) {
		return getComponentState(part.getIndex());
	}

	public Integer getComponentState(int partIndex) {
		ensureCapacity(partIndex);
		return marks[partIndex];
	}

	private void ensureCapacity(int partIndex) {
		if (partIndex >= marks.length) {
			int [] newmarks = new int[partIndex];
			System.arraycopy(marks, 0, newmarks, 0, marks.length);
			marks = newmarks;
		}		
	}

	public int getSize() {
		return marks.length;
	}

	public void removeFromComponentState(Indexable part, Integer s) {
		removeFromComponentState(part.getIndex(), s);
	}

	public void removeFromComponentState(int partIndex, Integer s) {
		ensureCapacity(partIndex);
		marks[partIndex] -= s ;		
	}

	public void setComponentState(Indexable part, Integer s) {
		setComponentState(part.getIndex(), s);
	}

	public void setComponentState(int partIndex, Integer s) {
		ensureCapacity(partIndex);
		marks[partIndex] = s;
	}


}
