package libpetri.structure.state;


import java.util.HashMap;

import libpetri.nets.INetType;


public class StateFactory {

	public static ComposedState<Integer> createPTState(int nbPlaces) {
		return new PTPlaceStateVector(nbPlaces);
	}

	
	/// Unique table mechanisms
	
	/// stores : net -> states set
	private static HashMap<INetType, HashMap<ComposedState,ComposedState>> states_ = new HashMap<INetType, HashMap<ComposedState,ComposedState>>();

	
	// Returns a state s' such that s==s', ensuring that the memory representation of s is unique
	public static ComposedState unique(ComposedState s, INetType netType) {

		// get reference to appropriate net type states set
		HashMap<ComposedState, ComposedState> st;
		synchronized (states_) {
			st = states_.get(netType);
			// first hit for this net type
			if (st == null) {
				st = new HashMap<ComposedState, ComposedState>();
				states_.put(netType, st);
			}
		}
		// lookup
		synchronized (st) {
			ComposedState concrete = st.get(s);
			if (concrete == null) {
				// new state
				st.put(s,s);
				return s;
			} else {
				// known state
				return concrete;
			}
		}
	}
	
	
	// for debug, pretty print of unique tables
	public static String toStringUniqueTable() {
		StringBuffer sb = new StringBuffer();
		sb.append("Unicity table : " + states_.size() + " net types seen.\n");
		for (INetType netType : states_.keySet()) {
			HashMap<ComposedState, ComposedState> states = states_.get(netType);
			sb.append(">>>Net type :"+netType.getName() + " states :"+ states.size()+"\n");
			int j = 0;
			for (ComposedState s : states.keySet()) {
				// would like s.toString(net), but which net ??? we only have the class and no instance.
				sb.append("\t"+j+++":"+s.toString()+"\n");
			}
		}		
		return sb.toString();		
	}
}
