package libpetri.test;

import java.util.Calendar;

import libpetri.structure.Net;
import libpetri.structure.Visibility;
import libpetri.structure.arcs.ArcType;

public class ColoredClientServer extends Net {

	public ColoredClientServer(String name) {
		super(name);

		// create a color class (typedef) clientID
		// create a color class (typedef) clientID
		StringBuffer clinames = new StringBuffer(32);
		for (int i=0; i < 4 ; i++)
			clinames.append("c"+i+",");
		clinames.deleteCharAt(clinames.length()-1);
		createTypeDef("clientID",clinames.toString());
				
		// creation of domain : clients
		createDomain("clients","clientID");

		createPlace("idle","clients");
		createPlace("wait","clients");
		createPlace("readAck","clients");
		createPlace("ack","clients");
		
		setPlaceState("idle","1'<c1> + 1'<c2> ");
		
		//		 create a color class (typedef) serverID
		// create a color class (typedef) serverID
		StringBuffer servnames = new StringBuffer(32);
		for (int i=0; i < 3 ; i++)
			servnames.append("s"+i+",");
		servnames.deleteCharAt(servnames.length()-1);
		createTypeDef("serverID",servnames.toString());
		
		//		 creation of domain : clients
		createDomain("servers","serverID");

		createPlace("idleServ","servers");
		createPlace("cleaning","servers");
		setPlaceState("idleServ","1'<s0> ");

		//		 creation of domain : clientsxserver
		createDomain("CxS","clientID,serverID");
		createPlace("req","CxS");
		createPlace("treat","CxS");
		
		
		
		String tname = "sendReq";
		createTransition(tname,Visibility.PRIVATE);
		createTransitionParameter(tname,"X","clientID");
		createTransitionParameter(tname,"Y","serverID");
		createArc("idle", tname, ArcType.INPUT, "1'<X>");
		createArc("wait", tname, ArcType.OUTPUT, "1'<X>");
		createArc("req", tname, ArcType.OUTPUT, "1'<X,Y>");
		
		
		tname = "getAck";
		createTransition(tname,Visibility.PRIVATE);
		createTransitionParameter(tname,"X","clientID");
		createArc("wait", tname, ArcType.INPUT, "1'<X>");
		createArc("ack", tname, ArcType.INPUT, "1'<X>");
		createArc("readAck", tname , ArcType.OUTPUT, "1'<X>");
		
		tname = "doOperation";
		createTransition(tname,Visibility.PRIVATE);
		createTransitionParameter(tname,"X","clientID");
		createArc("readAck", tname, ArcType.INPUT, "1'<X>");
		createArc("idle", tname, ArcType.OUTPUT, "1'<X>");

		
				
				
		tname = "getReq";
		createTransition(tname,Visibility.PRIVATE);
		createTransitionParameter(tname,"X","clientID");
		createTransitionParameter(tname,"Y","serverID");
		createArc("idleServ", tname, ArcType.INPUT, "1'<Y>");
		createArc("req", tname, ArcType.INPUT, "1'<X,Y>");
		createArc("treat", tname, ArcType.OUTPUT, "1'<X,Y>");
		
		tname = "sendAck";
		createTransition(tname,Visibility.PRIVATE);
		createTransitionParameter(tname,"Y","serverID");
		createTransitionParameter(tname,"X","clientID");
		createArc("treat", tname, ArcType.INPUT, "1'<X,Y>");
		createArc("ack", tname , ArcType.OUTPUT, "1'<X>");
		createArc("cleaning", tname , ArcType.OUTPUT, "1'<Y>");
		
		tname = "cleanup";
		createTransition(tname,Visibility.PRIVATE);
		createTransitionParameter(tname,"Y","serverID");
		createArc("cleaning", tname, ArcType.INPUT, "1'<Y>");
		createArc("idleServ", tname, ArcType.OUTPUT, "1'<Y>");
		
		
	}
	
	public static void main(String[] args) {
		ColoredClientServer n = new ColoredClientServer ("colored");
		{
			System.out.println(n);
			Calendar c = Calendar.getInstance();
			n.generateStateSpace();
			Calendar c2 = Calendar.getInstance();
			System.out.println(" elapsed : "+ (c2.getTimeInMillis()-c.getTimeInMillis())+"ms");
		}
	}

}
