package libpetri.test;

import java.util.Calendar;

import libpetri.structure.Net;
import libpetri.structure.Visibility;
import libpetri.structure.arcs.ArcType;

public class ColoredPrivateServer extends Net {

	public ColoredPrivateServer(String name, boolean isPrivate) {
		super(name);

		// create a color class (typedef) serverID
		StringBuffer servnames = new StringBuffer(32);
		for (int i=0; i < 3 ; i++)
			servnames.append("s"+i+",");
		servnames.deleteCharAt(servnames.length()-1);
		createTypeDef("serverID",servnames.toString());
				
		// creation of domain : clients
		createDomain("servers","serverID");

		createPlace("idleServ","servers");
		createPlace("treat","servers");
		createPlace("cleaning","servers");
		
		setPlaceState("idleServ","1'<s0> + 1'<s1>");
		
		String tname = "getReq";
		if (isPrivate)
			createTransition(tname,Visibility.PRIVATE);
		else
			createTransition(tname,Visibility.PUBLIC);
		createTransitionParameter(tname,"Y","serverID");
		createArc("idleServ", tname, ArcType.INPUT, "1'<Y>");
		createArc("treat", tname, ArcType.OUTPUT, "1'<Y>");
		
		tname = "sendAck";
		if (isPrivate)
			createTransition(tname,Visibility.PRIVATE);
		else
			createTransition(tname,Visibility.PUBLIC);
		createTransitionParameter(tname,"Y","serverID");
		createArc("treat", tname, ArcType.INPUT, "1'<Y>");
		createArc("cleaning", tname , ArcType.OUTPUT, "1'<Y>");
		
		tname = "cleanup";
		createTransition(tname,Visibility.PRIVATE);
		createTransitionParameter(tname,"Y","serverID");
		createArc("cleaning", tname, ArcType.INPUT, "1'<Y>");
		createArc("idleServ", tname, ArcType.OUTPUT, "1'<Y>");
	
	}
	
	public static void main(String[] args) {
		ColoredPrivateServer n = new ColoredPrivateServer ("colored",true);
		{
			System.out.println(n);
			Calendar c = Calendar.getInstance();
			n.generateStateSpace();
			Calendar c2 = Calendar.getInstance();
			System.out.println(" elapsed : "+ (c2.getTimeInMillis()-c.getTimeInMillis())+"ms");
		}
	}

}
