package transform.gui;

import gui.TableSorter;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;
import java.util.Map.Entry;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;

import libpetri.structure.Net;

import org.eclipse.uml2.uml.NamedElement;


public class TransformationProductsPanel extends JPanel  {

	@SuppressWarnings("unused")
	private boolean DEBUG = false;
	private HostListModel hlm;


	protected static String[] columnToolTips = {
		"The UML model source of the transformation",
		"The product obtained by transformation"
	};

	Vector<Entry<NamedElement, Net>> products;
	JTable configTable;

	public TransformationProductsPanel(Map<NamedElement, Net> products) {
		super(new BorderLayout());
		this.products  = new Vector<Entry<NamedElement,Net>>(products.entrySet());
		hlm = new HostListModel();	

		// for sorting by columns
		TableSorter sorter = new TableSorter(hlm);
		configTable = new JTable(sorter) {
			//			Implement table header = column tool tips.
			protected JTableHeader createDefaultTableHeader() {
				return new JTableHeader(columnModel) {
					public String getToolTipText(MouseEvent e) {
						java.awt.Point p = e.getPoint();
						int index = columnModel.getColumnIndexAtX(p.x);
						int realIndex = columnModel.getColumn(index).getModelIndex();
						return columnToolTips[realIndex];
					}
				};
			}
		};
		configTable.setDefaultRenderer(NamedElement.class, new ElementRenderer());
		sorter.setTableHeader(configTable.getTableHeader());

		configTable.setPreferredScrollableViewportSize(new Dimension(800, 70));	      			
		configTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION );

		JScrollPane scrollPane = new JScrollPane(configTable);
		add("Center",scrollPane);
	}

	public TransformationProductsPanel() {
		this( new HashMap<NamedElement, Net>());
	}

	public void updateConfiguration() {
		hlm.fireTableDataChanged();
	}

	class HostListModel extends AbstractTableModel {

		public static final int SOURCE_UML = 0;
		public static final int TRANSF_RESULT = 1;

		public Object getValueAt(int row, int col) {
			Entry<NamedElement, Net> p = products.get(row);
			switch (col) {
			case SOURCE_UML :
				return p.getKey().eClass().getName() + " : " + p.getKey().getName();
			case TRANSF_RESULT :
				return "Net :" + p.getValue().getName();
			}
			return "null";
		}

		public int getColumnCount() {
			return 2;
		}

		public int getRowCount() {
			return products.size();
		}

		public Class<?> getColumnClass(int columnIndex) {
			if (columnIndex == SOURCE_UML)
				return NamedElement.class;
			else
				return super.getColumnClass(columnIndex);
		}

		public String getColumnName(int c) {
			switch (c) {
			case SOURCE_UML :
				return "UML source";
			case TRANSF_RESULT :
				return "Transformation product";
			}
			return ("");
		}


		public boolean isCellEditable(int row, int col) {
			return false;
		}
	}
	public void setProducts(Map<NamedElement, Net> products) {
		this.products  = new Vector<Entry<NamedElement,Net>>(products.entrySet());
		hlm.fireTableDataChanged();
	}

	public Net getSelectedNet() {
		try {
			return products.get(configTable.getSelectedRow()).getValue();
		} catch (Exception e) {
			return products.get(0).getValue();
		}
	}

}
