package transform.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import transform.ServiceOrchestrator;

import libpetri.structure.Net;
import micropilot.Service;


public class UML2PetriGUI extends JFrame implements ActionListener {


	ServiceOrchestrator so;

	JPanel confGui;
	private JButton goButton ;
	private JButton transformButton;

	JTextArea results = new JTextArea(20,100);

	FKServiceRunner fksr = new FKServiceRunner();

	VerificationResultsPanel vrp = new VerificationResultsPanel();

	private static final String currentSavePath= "settings.stm2petri";


	private TransformationProductsPanel productsPanel;


	public UML2PetriGUI() {
		// frame init
		super("Behavioral Consistency Checker 1.0");
		setSize(1024,768);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// controller layer init 	
		
		JPanel top = new JPanel(new BorderLayout());
		so = new ServiceOrchestrator();
		try {
			so.loadConfiguration(currentSavePath);
		} catch (IOException e2) {
			System.err.println("No previously saved configuration found. Please use options->save settings.");
		}
		
		
		JPanel contents = new JPanel (new BorderLayout());
		confGui = so.getConf().getConfPanel();
		top.add("Center",confGui);
		

	//	JPanel fkPanel = new JPanel(); //= new JPanel(new GridLayout(1,2));

	//	fkPanel.add(so.getFkConf().getConfPanel());
	//  fkPanel.add(fksr);

	//	center.add(fkPanel);


		transformButton = new JButton("Transform");
		transformButton.setBackground(new Color(0,150,0,200));
		transformButton.addActionListener(this);
		top.add("South",transformButton);
		contents.add("North",top);
		
		JPanel center = new JPanel(new BorderLayout());		
		productsPanel = new TransformationProductsPanel(); 
		center.add("Center",productsPanel);

		
		goButton = new JButton("Check Consistency");
		goButton.setBackground(Color.green);
		goButton.addActionListener(this);
		goButton.setEnabled(false);
		center.add("South",goButton);
		
		contents.add("Center",center);
		
		JPanel res = new JPanel();
		//	res.add(new JScrollPane(results));
		res.add(vrp);
		contents.add("South",res);

		JMenuBar menu = new JMenuBar();
		JMenu options = new JMenu("Options");
		final JMenuItem mi_save = new JMenuItem ("Save settings");
		mi_save.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				Object source = e.getSource();
				if (source == mi_save) {
					try {
						so.saveConfiguration(currentSavePath);
					} catch (FileNotFoundException e1) {
						System.err.println( "Configuration save failed due bad file name error \n"+e1);
					} catch (IOException e3) {
						System.err.println( "Configuration save failed due to exception : " +e3);
					}
				}
			}
		});	
		options.add(mi_save);
		
		final JFrame frame = this;
		final JMenuItem mi_fkset = new JMenuItem ("CPNAMI settings");
		mi_fkset.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				Object source = e.getSource();
				if (source == mi_fkset) {
					JDialog di_fkset = new JDialog(frame,"CPNAMI Configuration settings");
					di_fkset.setSize(600,300);
					di_fkset.add(so.getFkConf().getConfPanel());
					di_fkset.setVisible(true);
				}
			}
		});	
		options.add(mi_fkset);
		
		menu.add(options);
		setJMenuBar(menu);



		setContentPane(contents);
		setVisible(true);
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == transformButton) {
			productsPanel.setProducts(so.doTransformation());
			results.append("Transformation to petri finished \n");
			if (productsPanel.products.size() != 0) {
				goButton.setEnabled(true);
			}
			repaint();
		} else if (e.getSource() == goButton) {
			Net n = productsPanel.getSelectedNet();
//			System.out.println("Selected net : " +n);
			Vector<Service> toCall = new Vector<Service>();
			toCall.add(Service.SYNTAX_CHECKER);
			toCall.add(Service.STRUCTURAL_BOUNDS);
			
			//vrp.setResults(so.callServiceList(n, fksr.getSelectedServices()));

			try {
				vrp.setResults(so.callServiceList(n, toCall));
			} catch (IOException e1) {
				JOptionPane.showMessageDialog(this, "Sorry. There has been an uncaught IO exception when running BCC.\n"+e1);
			}
			
			
//			results.append("IO Error :"+e +" \n while invoking service "+ s);
		}
	}



	public static void main(String[] args) {
		RedirectedFrame console = new RedirectedFrame(true,false,"",400,600,JFrame.DISPOSE_ON_CLOSE);
		UML2PetriGUI gui = new UML2PetriGUI();


	}

}
