package transform.gui;

import gui.TableSorter;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;

import micropilot.Level;

import org.eclipse.uml2.uml.NamedElement;

import transform.IVerificationResult;


public class VerificationResultsPanel  extends JPanel  {

	@SuppressWarnings("unused")
	private boolean DEBUG = false;
	private HostListModel hlm;


	protected static String[] columnToolTips = {
		"UML model element concerned",
		"Severity of the result",
		"Error description"
	};

	Vector<IVerificationResult> results;
	JTable configTable;

	public VerificationResultsPanel(Collection<IVerificationResult> vres) {
		super(new BorderLayout());
		this.results  = new Vector<IVerificationResult>(vres);
		hlm = new HostListModel();	

		// for sorting by columns
		TableSorter sorter = new TableSorter(hlm);
		configTable = new JTable(sorter) {
			//			Implement table header = column tool tips.
			protected JTableHeader createDefaultTableHeader() {
				return new JTableHeader(columnModel) {
					public String getToolTipText(MouseEvent e) {
						java.awt.Point p = e.getPoint();
						int index = columnModel.getColumnIndexAtX(p.x);
						int realIndex = columnModel.getColumn(index).getModelIndex();
						return columnToolTips[realIndex];
					}
				};
			}
		};             
		configTable.setDefaultRenderer(Level.class, new LevelRenderer());
		configTable.setDefaultRenderer(NamedElement.class, new ElementRenderer());
		sorter.setTableHeader(configTable.getTableHeader());

		//configTable.setPreferredScrollableViewportSize(new Dimension(800, 300));
		configTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION );

		configTable.getColumnModel().getColumn(0).setPreferredWidth(80);
		configTable.getColumnModel().getColumn(1).setPreferredWidth(150);
		configTable.getColumnModel().getColumn(2).setPreferredWidth(800);
		
		JScrollPane scrollPane = new JScrollPane(configTable,JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrollPane.setPreferredSize(new Dimension(1000, 350));
		add("Center",scrollPane);
	}

	public VerificationResultsPanel() {
		this( new Vector<IVerificationResult>());
	}

	public void updateConfiguration() {
		hlm.fireTableDataChanged();
	}

	class HostListModel extends AbstractTableModel {


		public static final int LEVEL = 0;
		public static final int SOURCE_ELEM = 1;
		public static final int DESC = 2;


		public Object getValueAt(int row, int col) {
			IVerificationResult p = results.get(row);
			switch (col) {
			case SOURCE_ELEM :
				return p.getTargetElement();
			case LEVEL :				
				return p.getSeverityLevel();
			case DESC :
				return p.getDescription();
			}
			return "null";
		}

		public Class<?> getColumnClass(int col) {
			switch (col) {
			case LEVEL : 
				return  Level.class ;
			case SOURCE_ELEM :
				return NamedElement.class;
			default :
				return String.class ; //super.getColumnClass(col);
			}
		}
		
		

		public int getColumnCount() {
			return 3;
		}

		public int getRowCount() {
			return results.size();
		}

		public String getColumnName(int c) {
			switch (c) {
			case SOURCE_ELEM :
				return "Location";
			case LEVEL :
				return "Severity";
			case DESC :
				return "Description";
			}
			return ("");
		}


		public boolean isCellEditable(int row, int col) {
			return false;
		}
	}
	public void setResults(Collection<IVerificationResult> products) {
		this.results  = new Vector<IVerificationResult>(products);
		hlm.fireTableDataChanged();
	}

	public IVerificationResult getSelectedResult() {
		return results.get(configTable.getSelectedRow());
	}

}
