package transform.stm;

import java.util.Map;

import libpetri.structure.Net;
import libpetri.structure.Visibility;
import libpetri.structure.arcs.ArcType;

import org.eclipse.uml2.uml.CallEvent;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.FinalState;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Pseudostate;
import org.eclipse.uml2.uml.PseudostateKind;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.StateMachine;
import org.eclipse.uml2.uml.Transition;
import org.eclipse.uml2.uml.Trigger;

import transform.Configuration;
import transform.Trace;

public class STMtoNetTransformer implements StateMachineTransformer {

	
	private Trace<Element, String> trace;
	private Map<NamedElement, Net> products;

	public STMtoNetTransformer(Trace<Element, String> trace, Map<NamedElement, Net> products) {
		this.trace = trace;
		this.products = products;
	}

	public void handleSTM(StateMachine stm, Configuration conf) {
		Net net = new Net(stm.getName());
		System.out.println("_______________"+net.getName());
		for (Element e : stm.allOwnedElements()) {
			System.out.println(e);
			String n = null;
			String mark = null;
			if (e instanceof Pseudostate) { 	
				n = ((Pseudostate)e).getKind().toString();
				PseudostateKind type = ((Pseudostate)e).getKind();
				if (type == PseudostateKind.INITIAL_LITERAL) {
					mark = "1'<>";
				}
			}
			else if (e instanceof FinalState) 
				n = "final";
			else if (e instanceof State) {
				State st = (State) e;
				n = st.getName();				
			} else if (e instanceof Transition) {
				continue ;
			}
			if (n == null) { 
				System.err.println("unparsed state : "+e);
				continue;
			} else {
				net.createPlace(n);
				if (mark != null)
					net.setPlaceState(n, mark);

				trace.addTrace(e, n);
			}

		}

		int trnum = 0;
		for (Element e : stm.allOwnedElements()) {
			if (e instanceof Transition) {
				Transition t = (Transition) e;
				String n = t.getName();
				Visibility v = Visibility.PRIVATE;
				if (n == null )
					n = "transition_"+trnum++ ;

				if (! t.isLeaf())
					for (Trigger trig : t.getTriggers() ) {
						System.out.println("trig : " +trig);
						if (trig.getEvent() instanceof CallEvent) {
							CallEvent call = (CallEvent) trig.getEvent();
							n = call.getOperation().getName();						
							v = Visibility.PUBLIC;
						}
					}

				n+= e.hashCode();
				net.createTransition(n,v);
				trace.addTrace(e, n);

				String src = trace.getTarget(t.getSource());
				String dest = trace.getTarget(t.getTarget());
				if (src != null) {
					net.createArc(src, n, ArcType.INPUT);
				}
				if (dest != null) {
					net.createArc(dest,n,ArcType.OUTPUT);
				}
			}
		}
		System.out.println("_______________");
		products.put(stm,net);
		//exhibit(net, conf);

	}

}
